@@ -3,41 +3,25 @@
  *@ for _header() versions: including "encoded word" as of RFC 2049):
  *@ - Quoted-Printable, section 6.7
  *@ - Base64, section 6.8
+ *@ QP quoting and _b64_decode(), b64_encode() inspired from NetBSDs mailx(1):
+ *@   $NetBSD: mime_codecs.c,v 1.9 2009/04/10 13:08:25 christos Exp $
  *@ TODO We have no notion of a "current message context" and thus badly log.
  *@ TODO This is not final yet, v15 will bring "filters".
  *
- * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  * Copyright (c) 2012 - 2018 Steffen (Daode) Nurpmeso <steffen@sdaoden.eu>.
- */
-/* QP quoting idea, _b64_decode(), b64_encode() taken from NetBSDs mailx(1): */
-/* $NetBSD: mime_codecs.c,v 1.9 2009/04/10 13:08:25 christos Exp $ */
-/*
- * Copyright (c) 2006 The NetBSD Foundation, Inc.
- * All rights reserved.
+ * SPDX-License-Identifier: ISC
  *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Anon Ymous.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 #undef n_FILE
 #define n_FILE mime_enc
@@ -507,7 +491,8 @@ qp_encode(struct str *out, struct str const *in, enum qpflags flags){
          out = NULL;
          goto jerr;
       }
-      out->s = (flags & QP_SALLOC) ? salloc(lnlen) : srealloc(out->s, lnlen);
+      out->s = (flags & QP_SALLOC) ? n_autorec_alloc(lnlen)
+            : n_realloc(out->s, lnlen);
    }
    qp = out->s;
    is = in->s;
@@ -762,7 +747,7 @@ jsoftnl:
          outrest->l = s.s_len;
          n_string_drop_ownership(sp);
          if(cp != NULL)
-            free(cp);
+            n_free(cp);
       }
       break;
    }
@@ -810,7 +795,8 @@ b64_encode(struct str *out, struct str const *in, enum b64flags flags){
          out = NULL;
          goto jleave;
       }
-      out->s = (flags & B64_SALLOC) ? salloc(i) : srealloc(out->s, i);
+      out->s = (flags & B64_SALLOC) ? n_autorec_alloc(i)
+            : n_realloc(out->s, i);
    }
    b64 = out->s;
 
@@ -929,9 +915,9 @@ b64_decode(struct str *out, struct str const *in){
 
    /* Ignore an empty input, as may happen for an empty final line */
    if(work.l == 0)
-      out->s = srealloc(out->s, 1);
+      out->s = n_realloc(out->s, 1);
    else if(work.l >= 4 && !(work.l & 3)){
-      out->s = srealloc(out->s, len +1);
+      out->s = n_realloc(out->s, len +1);
       if((ssize_t)(len = a_me_b64_decode(out, &work)) < 0)
          goto jerr;
    }else
@@ -958,9 +944,9 @@ b64_decode_header(struct str *out, struct str const *in){
          out = NULL;
 
       if(inr.s != NULL)
-         free(inr.s);
+         n_free(inr.s);
       if(outr.s != NULL)
-         free(outr.s);
+         n_free(outr.s);
    }
    NYD_LEAVE;
    return (out != NULL);
@@ -983,7 +969,7 @@ b64_decode_part(struct str *out, struct str const *in, struct str *outrest,
       if(len > 0)
          n_string_push_buf(&s, out->s, len);
       if(out->s != NULL)
-         free(out->s);
+         n_free(out->s);
    }
    out->s = NULL, out->l = 0;
    n_string_creat(&workbuf);
@@ -1094,7 +1080,7 @@ jrepl:
          if(b64l > 0 && b64l != 4){
             if(inrest_or_null == NULL)
                goto jerr;
-            inrest_or_null->s = srealloc(inrest_or_null->s, b64l +1);
+            inrest_or_null->s = n_realloc(inrest_or_null->s, b64l +1);
             inrest_or_null->s[0] = ca;
             if(b64l > 1)
                inrest_or_null->s[1] = cb;
