@@ -3,6 +3,7 @@
  *
  * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  * Copyright (c) 2012 - 2018 Steffen (Daode) Nurpmeso <sdaoden@users.sf.net>.
+ * SPDX-License-Identifier: BSD-4-Clause
  */
 /*-
  * Copyright (c) 1985, 1986, 1992, 1993
@@ -330,16 +331,16 @@ zfree(void *cookie)
 	zs = cookie;
 	if (zmode == 'w') {		/* Put out the final code. */
 		if (output(zs, (code_int) ent) == -1) {
-			free(zs);
+			n_free(zs);
 			return (-1);
 		}
 		out_count++;
 		if (output(zs, (code_int) - 1) == -1) {
-			free(zs);
+			n_free(zs);
 			return (-1);
 		}
 	}
-	free(zs);
+	n_free(zs);
 	return (0);
 }
 
@@ -691,7 +692,7 @@ zalloc(FILE *fp)
 #define bits	BITS
 	struct s_zstate *zs;
 
-	zs = scalloc(1, sizeof *zs);
+	zs = n_calloc(1, sizeof *zs);
 	maxbits = bits ? bits : BITS;	/* User settable max # bits/code. */
 	maxmaxcode = 1L << maxbits;	/* Should NEVER generate this code. */
 	hsize = HSIZE;			/* For dynamic table sizing. */
