@@ -9,6 +9,7 @@
  *@ Bug: in case of clashes of two-letter names terminfo(5) wins.
  *
  * Copyright (c) 2016 - 2018 Steffen (Daode) Nurpmeso <steffen@sdaoden.eu>.
+ * SPDX-License-Identifier: ISC
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -178,7 +179,7 @@ a_termcap_init_var(struct str const *termvar){
 
    assert(termvar->s[termvar->l] == '\0');
    i = termvar->l +1;
-   cbp_base = salloc(i);
+   cbp_base = n_autorec_alloc(i);
    memcpy(cbp = cbp_base, termvar->s, i);
 
    for(; (ccp = n_strsep(&cbp, ',', TRU1)) != NULL;){
@@ -222,7 +223,7 @@ jeinvent:
             if((f & a_TERMCAP_F_TYPE_MASK) == n_TERMCAP_CAPTYPE_STRING){
                struct a_termcap_ext_ent *teep;
 
-               teep = smalloc(n_VSTRUCT_SIZEOF(struct a_termcap_ext_ent,
+               teep = n_alloc(n_VSTRUCT_SIZEOF(struct a_termcap_ext_ent,
                      tee_name) + kl +1);
                teep->tee_next = a_termcap_g->tg_ext_ents;
                a_termcap_g->tg_ext_ents = teep;
@@ -267,7 +268,7 @@ jeinvent:
 #ifdef HAVE_KEY_BINDINGS
 jlearned:
 #endif
-      if(n_poption & n_PO_D_VV)
+      if(n_poption & n_PO_D_V)
          n_err(_("*termcap*: learned %.*s: %s\n"), (int)kl, ccp,
             (tep->te_flags & a_TERMCAP_F_DISABLED ? "<disabled>"
              : (f & a_TERMCAP_F_TYPE_MASK) == n_TERMCAP_CAPTYPE_BOOL ? "true"
@@ -390,15 +391,6 @@ a_termcap_init_altern(void){
       }
    }
 
-#ifdef HAVE_TERMCAP
-   /* cl == ho+cd */
-   tep = &a_termcap_g->tg_ents[n_TERMCAP_CMD_cl];
-   if(!a_OOK(tep)){
-      if(a_OK(n_TERMCAP_CMD_cd) && a_OK(n_TERMCAP_CMD_ho))
-         a_SET(tep, n_TERMCAP_CMD_cl, TRU1);
-   }
-#endif
-
 #ifdef HAVE_MLE
    /* ce == ch + [:SPACE:] (start column specified by argument) */
    tep = &a_termcap_g->tg_ents[n_TERMCAP_CMD_ce];
@@ -433,6 +425,15 @@ a_termcap_init_altern(void){
       tep->te_off = (ui16_t)a_termcap_g->tg_dat.s_len;
       n_string_push_buf(&a_termcap_g->tg_dat, "\033[C", sizeof("\033[C"));
    }
+
+# ifdef HAVE_TERMCAP
+   /* cl == ho+cd */
+   tep = &a_termcap_g->tg_ents[n_TERMCAP_CMD_cl];
+   if(!a_OOK(tep)){
+      if(a_OK(n_TERMCAP_CMD_cd) && a_OK(n_TERMCAP_CMD_ho))
+         a_SET(tep, n_TERMCAP_CMD_cl, TRU1);
+   }
+# endif
 #endif /* HAVE_MLE */
 
    NYD2_LEAVE;
@@ -616,7 +617,7 @@ n_termcap_init(void){
 
    assert((n_psonce & n_PSO_INTERACTIVE) && !(n_poption & n_PO_QUICKRUN_MASK));
 
-   a_termcap_g = smalloc(sizeof *a_termcap_g);
+   a_termcap_g = n_alloc(sizeof *a_termcap_g);
    a_termcap_g->tg_ext_ents = NULL;
    memset(&a_termcap_g->tg_ents[0], 0, sizeof(a_termcap_g->tg_ents));
    if((ccp = ok_vlook(termcap)) != NULL)
@@ -649,7 +650,7 @@ n_termcap_init(void){
             a_termcap_ent_query_tcp(tep, &a_termcap_control[i]);
       }
    }
-#endif
+#endif /* HAVE_TERMCAP */
 
    a_termcap_init_altern();
 
@@ -676,11 +677,11 @@ n_termcap_destroy(void){
 
       while((tmp = a_termcap_g->tg_ext_ents) != NULL){
          a_termcap_g->tg_ext_ents = tmp->tee_next;
-         free(tmp);
+         n_free(tmp);
       }
    }
    n_string_gut(&a_termcap_g->tg_dat);
-   free(a_termcap_g);
+   n_free(a_termcap_g);
    a_termcap_g = NULL;
 #endif
    NYD_LEAVE;
@@ -798,14 +799,6 @@ n_termcap_cmd(enum n_termcap_cmd cmd, ssize_t a1, ssize_t a2){
          rv = TRUM1;
          break;
 
-#ifdef HAVE_TERMCAP
-      case n_TERMCAP_CMD_cl: /* cl = ho + cd */
-         rv = n_termcap_cmdx(n_TERMCAP_CMD_ho);
-         if(rv > 0)
-            rv = n_termcap_cmdx(n_TERMCAP_CMD_cd | flags);
-         break;
-#endif
-
 #ifdef HAVE_MLE
       case n_TERMCAP_CMD_ce: /* ce == ch + [:SPACE:] */
          if(a1 > 0)
@@ -826,6 +819,13 @@ n_termcap_cmd(enum n_termcap_cmd cmd, ssize_t a1, ssize_t a2){
             rv = n_termcap_cmd(n_TERMCAP_CMD_nd, a1, -1);
          }
          break;
+# ifdef HAVE_TERMCAP
+      case n_TERMCAP_CMD_cl: /* cl = ho + cd */
+         rv = n_termcap_cmdx(n_TERMCAP_CMD_ho);
+         if(rv > 0)
+            rv = n_termcap_cmdx(n_TERMCAP_CMD_cd | flags);
+         break;
+# endif
 #endif /* HAVE_MLE */
       }
 
@@ -885,7 +885,7 @@ n_termcap_query(enum n_termcap_query query, struct n_termcap_value *tvp){
          goto jleave;
 #ifdef HAVE_TERMCAP
       nlen = strlen(ndat) +1;
-      teep = smalloc(n_VSTRUCT_SIZEOF(struct a_termcap_ext_ent, tee_name) +
+      teep = n_alloc(n_VSTRUCT_SIZEOF(struct a_termcap_ext_ent, tee_name) +
             nlen);
       tep = &teep->tee_super;
       teep->tee_next = a_termcap_g->tg_ext_ents;
